"""The socketserver commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SOCKETServer:ENAble {ON|OFF|<NR1>}
    - SOCKETServer:ENAble?
    - SOCKETServer:PORT <NR1>
    - SOCKETServer:PORT?
    - SOCKETServer:PROTOCol {TERMinal|NONe}
    - SOCKETServer:PROTOCol?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SocketserverProtocol(SCPICmdWrite, SCPICmdRead):
    """The ``SOCKETServer:PROTOCol`` command.

    **Description:**
        - This command sets or queries the protocol for the socket server.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOCKETServer:PROTOCol?`` query.
        - Using the ``.verify(value)`` method will send the ``SOCKETServer:PROTOCol?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOCKETServer:PROTOCol value`` command.

    **SCPI Syntax:**

    ::

        - SOCKETServer:PROTOCol {TERMinal|NONe}
        - SOCKETServer:PROTOCol?

    **Info:**
        - ``TERMinal`` specifies terminal protocol for the socket server. When set to TERMinal, a
          session startup message is sent to the socket and a command prompt is provided.
        - ``NONe`` disables the terminal features, allowing the server to be used for raw socket
          transactions, such as with a VISA socket server. The default setting is NONe.
    """


class SocketserverPort(SCPICmdWrite, SCPICmdRead):
    """The ``SOCKETServer:PORT`` command.

    **Description:**
        - This command sets the TCPIP port for the socket server connection.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOCKETServer:PORT?`` query.
        - Using the ``.verify(value)`` method will send the ``SOCKETServer:PORT?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOCKETServer:PORT value`` command.

    **SCPI Syntax:**

    ::

        - SOCKETServer:PORT <NR1>
        - SOCKETServer:PORT?

    **Info:**
        - ``<NR1>`` is the TCPIP port for the socket server connection.
    """


class SocketserverEnable(SCPICmdWrite, SCPICmdRead):
    """The ``SOCKETServer:ENAble`` command.

    **Description:**
        - This command enables or disables the socket server which supports a Telnet or other TCP/IP
          socket connection to send commands and queries to the instrument. The default state is
          enabled.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOCKETServer:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``SOCKETServer:ENAble?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOCKETServer:ENAble value`` command.

    **SCPI Syntax:**

    ::

        - SOCKETServer:ENAble {ON|OFF|<NR1>}
        - SOCKETServer:ENAble?

    **Info:**
        - ``ON`` enables the socket server.
        - ``OFF`` disables the socket server.
        - ``<NR1> = 0`` disables the socket server; any other value turns it on.
    """


class Socketserver(SCPICmdRead):
    """The ``SOCKETServer`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SOCKETServer?`` query.
        - Using the ``.verify(value)`` method will send the ``SOCKETServer?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.enable``: The ``SOCKETServer:ENAble`` command.
        - ``.port``: The ``SOCKETServer:PORT`` command.
        - ``.protocol``: The ``SOCKETServer:PROTOCol`` command.
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SOCKETServer"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._enable = SocketserverEnable(device, f"{self._cmd_syntax}:ENAble")
        self._port = SocketserverPort(device, f"{self._cmd_syntax}:PORT")
        self._protocol = SocketserverProtocol(device, f"{self._cmd_syntax}:PROTOCol")

    @property
    def enable(self) -> SocketserverEnable:
        """Return the ``SOCKETServer:ENAble`` command.

        **Description:**
            - This command enables or disables the socket server which supports a Telnet or other
              TCP/IP socket connection to send commands and queries to the instrument. The default
              state is enabled.

        **Usage:**
            - Using the ``.query()`` method will send the ``SOCKETServer:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``SOCKETServer:ENAble?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOCKETServer:ENAble value``
              command.

        **SCPI Syntax:**

        ::

            - SOCKETServer:ENAble {ON|OFF|<NR1>}
            - SOCKETServer:ENAble?

        **Info:**
            - ``ON`` enables the socket server.
            - ``OFF`` disables the socket server.
            - ``<NR1> = 0`` disables the socket server; any other value turns it on.
        """
        return self._enable

    @property
    def port(self) -> SocketserverPort:
        """Return the ``SOCKETServer:PORT`` command.

        **Description:**
            - This command sets the TCPIP port for the socket server connection.

        **Usage:**
            - Using the ``.query()`` method will send the ``SOCKETServer:PORT?`` query.
            - Using the ``.verify(value)`` method will send the ``SOCKETServer:PORT?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOCKETServer:PORT value`` command.

        **SCPI Syntax:**

        ::

            - SOCKETServer:PORT <NR1>
            - SOCKETServer:PORT?

        **Info:**
            - ``<NR1>`` is the TCPIP port for the socket server connection.
        """
        return self._port

    @property
    def protocol(self) -> SocketserverProtocol:
        """Return the ``SOCKETServer:PROTOCol`` command.

        **Description:**
            - This command sets or queries the protocol for the socket server.

        **Usage:**
            - Using the ``.query()`` method will send the ``SOCKETServer:PROTOCol?`` query.
            - Using the ``.verify(value)`` method will send the ``SOCKETServer:PROTOCol?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOCKETServer:PROTOCol value``
              command.

        **SCPI Syntax:**

        ::

            - SOCKETServer:PROTOCol {TERMinal|NONe}
            - SOCKETServer:PROTOCol?

        **Info:**
            - ``TERMinal`` specifies terminal protocol for the socket server. When set to TERMinal,
              a session startup message is sent to the socket and a command prompt is provided.
            - ``NONe`` disables the terminal features, allowing the server to be used for raw socket
              transactions, such as with a VISA socket server. The default setting is NONe.
        """
        return self._protocol



"""The output commands module.

These commands are used in the following models:
AWG70KA, AWG70KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - OUTPut[n]:ATTenuator:A1 <NR1>
    - OUTPut[n]:ATTenuator:A1?
    - OUTPut[n]:ATTenuator:A2 <NR1>
    - OUTPut[n]:ATTenuator:A2?
    - OUTPut[n]:ATTenuator:A3 <NR1>
    - OUTPut[n]:ATTenuator:A3?
    - OUTPut[n]:ATTenuator:DAC <NRf>
    - OUTPut[n]:ATTenuator:DAC?
    - OUTPut[n]:FILTer {NONE|BPASs|LPASs}
    - OUTPut[n]:FILTer:BPASs:RANGe {R10TO14GHZ|R13TO18GHZ}
    - OUTPut[n]:FILTer:BPASs:RANGe?
    - OUTPut[n]:FILTer?
    - OUTPut[n]:PATH {DIRect|AC|DCAmplified}
    - OUTPut[n]:PATH?
    - OUTPut[n]:STATe {0|1|OFF|ON}
    - OUTPut[n]:STATe?
    - OUTPut[n]:SVALue:ANALog:STATe {OFF|ZERO}
    - OUTPut[n]:SVALue:ANALog:STATe?
    - OUTPut[n]:SVALue:MARKer[m] {OFF|LOW}
    - OUTPut[n]:SVALue:MARKer[m]?
    - OUTPut[n]:WVALue:ANALog:STATe {FIRSt|ZERO}
    - OUTPut[n]:WVALue:ANALog:STATe?
    - OUTPut[n]:WVALue:MARKer[m] {FIRSt|LOW|HIGH}
    - OUTPut[n]:WVALue:MARKer[m]?
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class OutputItemWvalueMarkerItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:WVALue:MARKer[m]`` command.

    **Description:**
        - This command sets or returns the output condition of the specified marker of the specified
          channel while the instrument is in the waiting-for-trigger state or for a brief period
          after the waveform loads to the DAC and before the first point plays.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue:MARKer[m]?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue:MARKer[m]?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:WVALue:MARKer[m] value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:WVALue:MARKer[m] {FIRSt|LOW|HIGH}
        - OUTPut[n]:WVALue:MARKer[m]?

    **Info:**
        - ``FIRSt`` sets the marker output level to match the first point in the waveform when the
          channel is in the waiting-for-trigger state. LOW sets the marker output to a logic level
          low for when the channel is in the waiting-for-trigger state. HIGH sets the marker output
          to a logic level high when the channel is in the waiting-for-trigger state.[n] determines
          the channel number. If omitted, interpreted as 1.[m] determines the marker number. If
          omitted, interpreted as 1.
        - ``*RST`` sets all channels to LOW.
    """


class OutputItemWvalueAnalogState(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:WVALue:ANALog:STATe`` command.

    **Description:**
        - This command sets or returns the output condition of a waveform of the specified channel
          while the instrument is in the waiting-for-trigger state or for a brief period after the
          waveform loads to the DAC and before the first point plays.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue:ANALog:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue:ANALog:STATe?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:WVALue:ANALog:STATe value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:WVALue:ANALog:STATe {FIRSt|ZERO}
        - OUTPut[n]:WVALue:ANALog:STATe?

    **Info:**
        - ``FIRSt`` sets the output level for channel 'n' to match the first point in the waveform
          when channel 'n' is in the Waiting-for-trigger state. ZERO sets the output level for
          channel 'n' to 0 volts when channel 'n' is in the Waiting-for-trigger state.[n] determines
          the channel number. If omitted, interpreted as 1.
        - ``*RST`` sets all channels to ZERO.
    """


class OutputItemWvalueAnalog(SCPICmdRead):
    """The ``OUTPut[n]:WVALue:ANALog`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue:ANALog?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue:ANALog?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``OUTPut[n]:WVALue:ANALog:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = OutputItemWvalueAnalogState(device, f"{self._cmd_syntax}:STATe")

    @property
    def state(self) -> OutputItemWvalueAnalogState:
        """Return the ``OUTPut[n]:WVALue:ANALog:STATe`` command.

        **Description:**
            - This command sets or returns the output condition of a waveform of the specified
              channel while the instrument is in the waiting-for-trigger state or for a brief period
              after the waveform loads to the DAC and before the first point plays.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue:ANALog:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue:ANALog:STATe?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``OUTPut[n]:WVALue:ANALog:STATe value`` command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:WVALue:ANALog:STATe {FIRSt|ZERO}
            - OUTPut[n]:WVALue:ANALog:STATe?

        **Info:**
            - ``FIRSt`` sets the output level for channel 'n' to match the first point in the
              waveform when channel 'n' is in the Waiting-for-trigger state. ZERO sets the output
              level for channel 'n' to 0 volts when channel 'n' is in the Waiting-for-trigger
              state.[n] determines the channel number. If omitted, interpreted as 1.
            - ``*RST`` sets all channels to ZERO.
        """
        return self._state


class OutputItemWvalue(SCPICmdRead):
    """The ``OUTPut[n]:WVALue`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.marker``: The ``OUTPut[n]:WVALue:MARKer[m]`` command.
        - ``.analog``: The ``OUTPut[n]:WVALue:ANALog`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._marker: Dict[int, OutputItemWvalueMarkerItem] = DefaultDictPassKeyToFactory(
            lambda x: OutputItemWvalueMarkerItem(device, f"{self._cmd_syntax}:MARKer{x}")
        )
        self._analog = OutputItemWvalueAnalog(device, f"{self._cmd_syntax}:ANALog")

    @property
    def marker(self) -> Dict[int, OutputItemWvalueMarkerItem]:
        """Return the ``OUTPut[n]:WVALue:MARKer[m]`` command.

        **Description:**
            - This command sets or returns the output condition of the specified marker of the
              specified channel while the instrument is in the waiting-for-trigger state or for a
              brief period after the waveform loads to the DAC and before the first point plays.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue:MARKer[m]?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue:MARKer[m]?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:WVALue:MARKer[m] value``
              command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:WVALue:MARKer[m] {FIRSt|LOW|HIGH}
            - OUTPut[n]:WVALue:MARKer[m]?

        **Info:**
            - ``FIRSt`` sets the marker output level to match the first point in the waveform when
              the channel is in the waiting-for-trigger state. LOW sets the marker output to a logic
              level low for when the channel is in the waiting-for-trigger state. HIGH sets the
              marker output to a logic level high when the channel is in the waiting-for-trigger
              state.[n] determines the channel number. If omitted, interpreted as 1.[m] determines
              the marker number. If omitted, interpreted as 1.
            - ``*RST`` sets all channels to LOW.
        """
        return self._marker

    @property
    def analog(self) -> OutputItemWvalueAnalog:
        """Return the ``OUTPut[n]:WVALue:ANALog`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue:ANALog?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue:ANALog?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``OUTPut[n]:WVALue:ANALog:STATe`` command.
        """
        return self._analog


class OutputItemSvalueMarkerItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:SVALue:MARKer[m]`` command.

    **Description:**
        - This command sets or returns the output data position of the specified marker of the
          specified channel when in the stopped state.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue:MARKer[m]?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue:MARKer[m]?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:SVALue:MARKer[m] value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:SVALue:MARKer[m] {OFF|LOW}
        - OUTPut[n]:SVALue:MARKer[m]?

    **Info:**
        - ``OFF`` sets the stop state marker output for channel 'n' to open (electrically
          disconnected). LOW sets the stop state marker output for channel 'n' value to 0 volts.[n]
          determines the channel number. If omitted, interpreted as 1.[m] determines the marker
          number. If omitted, interpreted as 1.
        - ``*RST`` sets all channel markers to LOW.
    """


class OutputItemSvalueAnalogState(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:SVALue:ANALog:STATe`` command.

    **Description:**
        - This command sets or returns the output condition of a waveform of the specified channel
          while the instrument is in the stopped state.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue:ANALog:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue:ANALog:STATe?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:SVALue:ANALog:STATe value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:SVALue:ANALog:STATe {OFF|ZERO}
        - OUTPut[n]:SVALue:ANALog:STATe?

    **Info:**
        - ``OFF`` sets the stop state output for channel 'n' to open (electrically disconnected).
          ZERO sets the stop state output value for channel 'n' to 0 volts.[n] determines the
          channel number. If omitted, interpreted as 1.
        - ``*RST`` sets all channels to ZERO.
    """


class OutputItemSvalueAnalog(SCPICmdRead):
    """The ``OUTPut[n]:SVALue:ANALog`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue:ANALog?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue:ANALog?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``OUTPut[n]:SVALue:ANALog:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = OutputItemSvalueAnalogState(device, f"{self._cmd_syntax}:STATe")

    @property
    def state(self) -> OutputItemSvalueAnalogState:
        """Return the ``OUTPut[n]:SVALue:ANALog:STATe`` command.

        **Description:**
            - This command sets or returns the output condition of a waveform of the specified
              channel while the instrument is in the stopped state.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue:ANALog:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue:ANALog:STATe?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``OUTPut[n]:SVALue:ANALog:STATe value`` command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:SVALue:ANALog:STATe {OFF|ZERO}
            - OUTPut[n]:SVALue:ANALog:STATe?

        **Info:**
            - ``OFF`` sets the stop state output for channel 'n' to open (electrically
              disconnected). ZERO sets the stop state output value for channel 'n' to 0 volts.[n]
              determines the channel number. If omitted, interpreted as 1.
            - ``*RST`` sets all channels to ZERO.
        """
        return self._state


class OutputItemSvalue(SCPICmdRead):
    """The ``OUTPut[n]:SVALue`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.marker``: The ``OUTPut[n]:SVALue:MARKer[m]`` command.
        - ``.analog``: The ``OUTPut[n]:SVALue:ANALog`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._marker: Dict[int, OutputItemSvalueMarkerItem] = DefaultDictPassKeyToFactory(
            lambda x: OutputItemSvalueMarkerItem(device, f"{self._cmd_syntax}:MARKer{x}")
        )
        self._analog = OutputItemSvalueAnalog(device, f"{self._cmd_syntax}:ANALog")

    @property
    def marker(self) -> Dict[int, OutputItemSvalueMarkerItem]:
        """Return the ``OUTPut[n]:SVALue:MARKer[m]`` command.

        **Description:**
            - This command sets or returns the output data position of the specified marker of the
              specified channel when in the stopped state.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue:MARKer[m]?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue:MARKer[m]?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:SVALue:MARKer[m] value``
              command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:SVALue:MARKer[m] {OFF|LOW}
            - OUTPut[n]:SVALue:MARKer[m]?

        **Info:**
            - ``OFF`` sets the stop state marker output for channel 'n' to open (electrically
              disconnected). LOW sets the stop state marker output for channel 'n' value to 0
              volts.[n] determines the channel number. If omitted, interpreted as 1.[m] determines
              the marker number. If omitted, interpreted as 1.
            - ``*RST`` sets all channel markers to LOW.
        """
        return self._marker

    @property
    def analog(self) -> OutputItemSvalueAnalog:
        """Return the ``OUTPut[n]:SVALue:ANALog`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue:ANALog?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue:ANALog?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``OUTPut[n]:SVALue:ANALog:STATe`` command.
        """
        return self._analog


class OutputItemState(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:STATe`` command.

    **Description:**
        - This command sets or returns the output state of the specified channel.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:STATe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:STATe value`` command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:STATe {0|1|OFF|ON}
        - OUTPut[n]:STATe?

    **Info:**
        - ``0`` or OFF disables the channel's output. 1 or ON enables the channel's output.[n]
          determines the channel number. If omitted, interpreted as 1.
        - ``*RST`` sets all channels to 0.
    """


class OutputItemPath(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:PATH`` command.

    **Description:**
        - This command sets or returns the output signal path of the specified channel. The command
          is only valid when Option AC is installed (AWG70001 only) or an MDC4500 is connected to
          the AWG70000 Series instrument.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:PATH?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:PATH?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:PATH value`` command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:PATH {DIRect|AC|DCAmplified}
        - OUTPut[n]:PATH?

    **Info:**
        - ``DIRect`` sets the signal path to go directly from the DAC to the + and - differential
          outputs. AC sets signal path to go through the attenuators and amplifiers, then to the
          single-ended AC output. Available only for the AWG70001 with Option AC.DCAmplified sets
          the signal path to go directly from the DAC to the + and - differential outputs that must
          be connected to the inputs of an MDC4500. The DCAmplified setting provides additional
          output range and output offset adjustment from the MDC4500 outputs.
    """


class OutputItemFilterBpassRange(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:FILTer:BPASs:RANGe`` command.

    **Description:**
        - This command sets or returns the filter range of the band pass filter for the signal path.
          Option AC is required. This command does not change the filter hardware cutoff
          frequencies, but selects an already available filter range with the specified cutoff
          frequencies. If Option AC is not installed, sending the command causes an error message.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:FILTer:BPASs:RANGe?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:FILTer:BPASs:RANGe?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:FILTer:BPASs:RANGe value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:FILTer:BPASs:RANGe {R10TO14GHZ|R13TO18GHZ}
        - OUTPut[n]:FILTer:BPASs:RANGe?
    """


class OutputItemFilterBpass(SCPICmdRead):
    """The ``OUTPut[n]:FILTer:BPASs`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:FILTer:BPASs?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:FILTer:BPASs?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.range``: The ``OUTPut[n]:FILTer:BPASs:RANGe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._range = OutputItemFilterBpassRange(device, f"{self._cmd_syntax}:RANGe")

    @property
    def range(self) -> OutputItemFilterBpassRange:
        """Return the ``OUTPut[n]:FILTer:BPASs:RANGe`` command.

        **Description:**
            - This command sets or returns the filter range of the band pass filter for the signal
              path. Option AC is required. This command does not change the filter hardware cutoff
              frequencies, but selects an already available filter range with the specified cutoff
              frequencies. If Option AC is not installed, sending the command causes an error
              message.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:FILTer:BPASs:RANGe?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:FILTer:BPASs:RANGe?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``OUTPut[n]:FILTer:BPASs:RANGe value`` command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:FILTer:BPASs:RANGe {R10TO14GHZ|R13TO18GHZ}
            - OUTPut[n]:FILTer:BPASs:RANGe?
        """
        return self._range


class OutputItemFilter(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:FILTer`` command.

    **Description:**
        - This command sets or returns the filter type for the signal path. Option AC is required.
          If Option AC is not installed, sending the command causes an error message.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:FILTer?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:FILTer?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:FILTer value`` command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:FILTer {NONE|BPASs|LPASs}
        - OUTPut[n]:FILTer?

    Properties:
        - ``.bpass``: The ``OUTPut[n]:FILTer:BPASs`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bpass = OutputItemFilterBpass(device, f"{self._cmd_syntax}:BPASs")

    @property
    def bpass(self) -> OutputItemFilterBpass:
        """Return the ``OUTPut[n]:FILTer:BPASs`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:FILTer:BPASs?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:FILTer:BPASs?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.range``: The ``OUTPut[n]:FILTer:BPASs:RANGe`` command.
        """
        return self._bpass


class OutputItemAttenuatorDac(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:ATTenuator:DAC`` command.

    **Description:**
        - This command sets or returns the DAC output power level (in units of dBm) when the Channel
          1 Output is set to AC. Option AC is required. If Option AC is not installed, sending the
          command causes an error message. See a diagram of the AC output path in the instrument's
          User Interface under Setup>Channel1>Advanced.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:DAC?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:DAC?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:DAC value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:ATTenuator:DAC <NRf>
        - OUTPut[n]:ATTenuator:DAC?
    """


class OutputItemAttenuatorA3(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:ATTenuator:A3`` command.

    **Description:**
        - This command sets or returns the step value setting for the A3 attenuator (in units of dB)
          in the Band Pass Filter path when the Channel 1 Output Path is set to AC. Option AC is
          required. If Option AC is not installed, sending the command causes an error message. See
          a diagram of the AC output path in the instrument's User Interface under
          Setup>Channel1>Advanced.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:A3?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:A3?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:A3 value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:ATTenuator:A3 <NR1>
        - OUTPut[n]:ATTenuator:A3?
    """


class OutputItemAttenuatorA2(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:ATTenuator:A2`` command.

    **Description:**
        - This command sets or returns the step value setting for the A2 attenuator (in units of dB)
          in the Band Pass Filter path when the Channel 1 Output Path is set to AC. Option AC is
          required. If Option AC is not installed, sending the command causes an error message. See
          a diagram of the AC output path in the instrument's User Interface under
          Setup>Channel1>Advanced.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:A2?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:A2?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:A2 value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:ATTenuator:A2 <NR1>
        - OUTPut[n]:ATTenuator:A2?
    """


class OutputItemAttenuatorA1(SCPICmdWrite, SCPICmdRead):
    """The ``OUTPut[n]:ATTenuator:A1`` command.

    **Description:**
        - This command sets or returns the step value setting for the A1 attenuator in (units of dB)
          in the Band Pass Filter path when the Channel 1 Output Path is set to AC and the Band Pass
          Range is set to 13 - 18 GHz. Option AC is required. If Option AC is not installed, sending
          the command causes an error message. See a diagram of the AC output path in the
          instrument's User Interface under Setup>Channel1>Advanced.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:A1?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:A1?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:A1 value``
          command.

    **SCPI Syntax:**

    ::

        - OUTPut[n]:ATTenuator:A1 <NR1>
        - OUTPut[n]:ATTenuator:A1?
    """


class OutputItemAttenuator(SCPICmdRead):
    """The ``OUTPut[n]:ATTenuator`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.a1``: The ``OUTPut[n]:ATTenuator:A1`` command.
        - ``.a2``: The ``OUTPut[n]:ATTenuator:A2`` command.
        - ``.a3``: The ``OUTPut[n]:ATTenuator:A3`` command.
        - ``.dac``: The ``OUTPut[n]:ATTenuator:DAC`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._a1 = OutputItemAttenuatorA1(device, f"{self._cmd_syntax}:A1")
        self._a2 = OutputItemAttenuatorA2(device, f"{self._cmd_syntax}:A2")
        self._a3 = OutputItemAttenuatorA3(device, f"{self._cmd_syntax}:A3")
        self._dac = OutputItemAttenuatorDac(device, f"{self._cmd_syntax}:DAC")

    @property
    def a1(self) -> OutputItemAttenuatorA1:
        """Return the ``OUTPut[n]:ATTenuator:A1`` command.

        **Description:**
            - This command sets or returns the step value setting for the A1 attenuator in (units of
              dB) in the Band Pass Filter path when the Channel 1 Output Path is set to AC and the
              Band Pass Range is set to 13 - 18 GHz. Option AC is required. If Option AC is not
              installed, sending the command causes an error message. See a diagram of the AC output
              path in the instrument's User Interface under Setup>Channel1>Advanced.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:A1?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:A1?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:A1 value``
              command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:ATTenuator:A1 <NR1>
            - OUTPut[n]:ATTenuator:A1?
        """
        return self._a1

    @property
    def a2(self) -> OutputItemAttenuatorA2:
        """Return the ``OUTPut[n]:ATTenuator:A2`` command.

        **Description:**
            - This command sets or returns the step value setting for the A2 attenuator (in units of
              dB) in the Band Pass Filter path when the Channel 1 Output Path is set to AC. Option
              AC is required. If Option AC is not installed, sending the command causes an error
              message. See a diagram of the AC output path in the instrument's User Interface under
              Setup>Channel1>Advanced.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:A2?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:A2?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:A2 value``
              command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:ATTenuator:A2 <NR1>
            - OUTPut[n]:ATTenuator:A2?
        """
        return self._a2

    @property
    def a3(self) -> OutputItemAttenuatorA3:
        """Return the ``OUTPut[n]:ATTenuator:A3`` command.

        **Description:**
            - This command sets or returns the step value setting for the A3 attenuator (in units of
              dB) in the Band Pass Filter path when the Channel 1 Output Path is set to AC. Option
              AC is required. If Option AC is not installed, sending the command causes an error
              message. See a diagram of the AC output path in the instrument's User Interface under
              Setup>Channel1>Advanced.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:A3?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:A3?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:A3 value``
              command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:ATTenuator:A3 <NR1>
            - OUTPut[n]:ATTenuator:A3?
        """
        return self._a3

    @property
    def dac(self) -> OutputItemAttenuatorDac:
        """Return the ``OUTPut[n]:ATTenuator:DAC`` command.

        **Description:**
            - This command sets or returns the DAC output power level (in units of dBm) when the
              Channel 1 Output is set to AC. Option AC is required. If Option AC is not installed,
              sending the command causes an error message. See a diagram of the AC output path in
              the instrument's User Interface under Setup>Channel1>Advanced.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator:DAC?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator:DAC?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:ATTenuator:DAC value``
              command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:ATTenuator:DAC <NRf>
            - OUTPut[n]:ATTenuator:DAC?
        """
        return self._dac


class OutputItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``OUTPut[n]`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``OUTPut[n]?`` query.
        - Using the ``.verify(value)`` method will send the ``OUTPut[n]?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.attenuator``: The ``OUTPut[n]:ATTenuator`` command tree.
        - ``.filter``: The ``OUTPut[n]:FILTer`` command.
        - ``.path``: The ``OUTPut[n]:PATH`` command.
        - ``.svalue``: The ``OUTPut[n]:SVALue`` command tree.
        - ``.wvalue``: The ``OUTPut[n]:WVALue`` command tree.
        - ``.state``: The ``OUTPut[n]:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "OUTPut[n]") -> None:
        super().__init__(device, cmd_syntax)
        self._attenuator = OutputItemAttenuator(device, f"{self._cmd_syntax}:ATTenuator")
        self._filter = OutputItemFilter(device, f"{self._cmd_syntax}:FILTer")
        self._path = OutputItemPath(device, f"{self._cmd_syntax}:PATH")
        self._svalue = OutputItemSvalue(device, f"{self._cmd_syntax}:SVALue")
        self._wvalue = OutputItemWvalue(device, f"{self._cmd_syntax}:WVALue")
        self._state = OutputItemState(device, f"{self._cmd_syntax}:STATe")

    @property
    def attenuator(self) -> OutputItemAttenuator:
        """Return the ``OUTPut[n]:ATTenuator`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:ATTenuator?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:ATTenuator?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.a1``: The ``OUTPut[n]:ATTenuator:A1`` command.
            - ``.a2``: The ``OUTPut[n]:ATTenuator:A2`` command.
            - ``.a3``: The ``OUTPut[n]:ATTenuator:A3`` command.
            - ``.dac``: The ``OUTPut[n]:ATTenuator:DAC`` command.
        """
        return self._attenuator

    @property
    def filter(self) -> OutputItemFilter:
        """Return the ``OUTPut[n]:FILTer`` command.

        **Description:**
            - This command sets or returns the filter type for the signal path. Option AC is
              required. If Option AC is not installed, sending the command causes an error message.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:FILTer?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:FILTer?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:FILTer value`` command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:FILTer {NONE|BPASs|LPASs}
            - OUTPut[n]:FILTer?

        Sub-properties:
            - ``.bpass``: The ``OUTPut[n]:FILTer:BPASs`` command tree.
        """
        return self._filter

    @property
    def path(self) -> OutputItemPath:
        """Return the ``OUTPut[n]:PATH`` command.

        **Description:**
            - This command sets or returns the output signal path of the specified channel. The
              command is only valid when Option AC is installed (AWG70001 only) or an MDC4500 is
              connected to the AWG70000 Series instrument.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:PATH?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:PATH?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:PATH value`` command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:PATH {DIRect|AC|DCAmplified}
            - OUTPut[n]:PATH?

        **Info:**
            - ``DIRect`` sets the signal path to go directly from the DAC to the + and -
              differential outputs. AC sets signal path to go through the attenuators and
              amplifiers, then to the single-ended AC output. Available only for the AWG70001 with
              Option AC.DCAmplified sets the signal path to go directly from the DAC to the + and -
              differential outputs that must be connected to the inputs of an MDC4500. The
              DCAmplified setting provides additional output range and output offset adjustment from
              the MDC4500 outputs.
        """
        return self._path

    @property
    def svalue(self) -> OutputItemSvalue:
        """Return the ``OUTPut[n]:SVALue`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:SVALue?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:SVALue?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.marker``: The ``OUTPut[n]:SVALue:MARKer[m]`` command.
            - ``.analog``: The ``OUTPut[n]:SVALue:ANALog`` command tree.
        """
        return self._svalue

    @property
    def wvalue(self) -> OutputItemWvalue:
        """Return the ``OUTPut[n]:WVALue`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:WVALue?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:WVALue?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.marker``: The ``OUTPut[n]:WVALue:MARKer[m]`` command.
            - ``.analog``: The ``OUTPut[n]:WVALue:ANALog`` command tree.
        """
        return self._wvalue

    @property
    def state(self) -> OutputItemState:
        """Return the ``OUTPut[n]:STATe`` command.

        **Description:**
            - This command sets or returns the output state of the specified channel.

        **Usage:**
            - Using the ``.query()`` method will send the ``OUTPut[n]:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``OUTPut[n]:STATe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``OUTPut[n]:STATe value`` command.

        **SCPI Syntax:**

        ::

            - OUTPut[n]:STATe {0|1|OFF|ON}
            - OUTPut[n]:STATe?

        **Info:**
            - ``0`` or OFF disables the channel's output. 1 or ON enables the channel's output.[n]
              determines the channel number. If omitted, interpreted as 1.
            - ``*RST`` sets all channels to 0.
        """
        return self._state



# pylint: disable=too-many-lines
# pylint: disable=line-too-long
"""The afg commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - AFG:AMPLitude <NR3>
    - AFG:AMPLitude?
    - AFG:ARBitrary:ARB<x>:DATE?
    - AFG:ARBitrary:ARB<x>:LABel <QString>
    - AFG:ARBitrary:ARB<x>:LABel?
    - AFG:ARBitrary:ARB<x>:TIMe?
    - AFG:ARBitrary:EMEM:FUNCtion?
    - AFG:ARBitrary:EMEM:GENerate {SINE|SQUare|PULSe|RAMP|NOISe[,NR1]}
    - AFG:ARBitrary:EMEM:GENerate?
    - AFG:ARBitrary:EMEM:NUMPoints?
    - AFG:ARBitrary:EMEM:POINTS <BlockWfmInDTO> |<NrfWfmInDTO>
    - AFG:ARBitrary:EMEM:POINTS:BYTEORDer <LSB> |<MSB>
    - AFG:ARBitrary:EMEM:POINTS:BYTEORDer?
    - AFG:ARBitrary:EMEM:POINTS:ENCdg {ASCii|BINary}
    - AFG:ARBitrary:EMEM:POINTS:ENCdg?
    - AFG:ARBitrary:EMEM:POINTS?
    - AFG:FREQuency <NR3>
    - AFG:FREQuency?
    - AFG:FUNCtion {SINE|SQUare|PULSe|RAMP|NOISe|DC|SINC|GAUSsian|LORENtz |ERISe|EDECAy|HAVERSINe|CARDIac|ARBitrary}
    - AFG:FUNCtion?
    - AFG:HIGHLevel <NR3>
    - AFG:HIGHLevel?
    - AFG:LEVELPreset {CMOS_5_0V|CMOS_3_3V|CMOS_2_5V|ECL|TTL|USER}
    - AFG:LEVELPreset?
    - AFG:LOWLevel <NR3>
    - AFG:LOWLevel?
    - AFG:NOISEAdd:PERCent <NR3>
    - AFG:NOISEAdd:PERCent?
    - AFG:NOISEAdd:STATE {0|1|OFF|ON}
    - AFG:NOISEAdd:STATE?
    - AFG:OFFSet <NR3>
    - AFG:OFFSet?
    - AFG:OUTPut:LOAd:IMPEDance {FIFty|HIGHZ}
    - AFG:OUTPut:LOAd:IMPEDance?
    - AFG:OUTPut:STATE {0|1|OFF|ON}
    - AFG:OUTPut:STATE?
    - AFG:PERIod <NR3>
    - AFG:PERIod?
    - AFG:PHASe <NR3>
    - AFG:PHASe?
    - AFG:PULse:WIDth <NR3>
    - AFG:PULse:WIDth?
    - AFG:RAMP:SYMmetry <NR3>
    - AFG:RAMP:SYMmetry?
    - AFG:SQUare:DUty <NR3>
    - AFG:SQUare:DUty?
"""  # noqa: E501
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class AfgSquareDuty(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:SQUare:DUty`` command.

    **Description:**
        - Sets (or queries) the AFG duty cycle in percent. The minimum is 10.0%, maximum is 90.0%
          and increment is 0.10%.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:SQUare:DUty?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:SQUare:DUty?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:SQUare:DUty value`` command.

    **SCPI Syntax:**

    ::

        - AFG:SQUare:DUty <NR3>
        - AFG:SQUare:DUty?

    **Info:**
        - ``<NR3>`` is the floating point number that represents the AFG duty cycle, as a
          percentage.
    """


class AfgSquare(SCPICmdRead):
    """The ``AFG:SQUare`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:SQUare?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:SQUare?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.duty``: The ``AFG:SQUare:DUty`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._duty = AfgSquareDuty(device, f"{self._cmd_syntax}:DUty")

    @property
    def duty(self) -> AfgSquareDuty:
        """Return the ``AFG:SQUare:DUty`` command.

        **Description:**
            - Sets (or queries) the AFG duty cycle in percent. The minimum is 10.0%, maximum is
              90.0% and increment is 0.10%.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:SQUare:DUty?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:SQUare:DUty?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:SQUare:DUty value`` command.

        **SCPI Syntax:**

        ::

            - AFG:SQUare:DUty <NR3>
            - AFG:SQUare:DUty?

        **Info:**
            - ``<NR3>`` is the floating point number that represents the AFG duty cycle, as a
              percentage.
        """
        return self._duty


class AfgRampSymmetry(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:RAMP:SYMmetry`` command.

    **Description:**
        - Sets (or queries) the AFG ramp symmetry in percent. Minimum is 0.0%, maximum is 100.0% and
          increment is 0.10%.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:RAMP:SYMmetry?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:RAMP:SYMmetry?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:RAMP:SYMmetry value`` command.

    **SCPI Syntax:**

    ::

        - AFG:RAMP:SYMmetry <NR3>
        - AFG:RAMP:SYMmetry?

    **Info:**
        - ``<NR3>`` is the floating point number that represents the AFG ramp symmetry, as a
          percentage.
    """


class AfgRamp(SCPICmdRead):
    """The ``AFG:RAMP`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:RAMP?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:RAMP?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.symmetry``: The ``AFG:RAMP:SYMmetry`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._symmetry = AfgRampSymmetry(device, f"{self._cmd_syntax}:SYMmetry")

    @property
    def symmetry(self) -> AfgRampSymmetry:
        """Return the ``AFG:RAMP:SYMmetry`` command.

        **Description:**
            - Sets (or queries) the AFG ramp symmetry in percent. Minimum is 0.0%, maximum is 100.0%
              and increment is 0.10%.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:RAMP:SYMmetry?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:RAMP:SYMmetry?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:RAMP:SYMmetry value`` command.

        **SCPI Syntax:**

        ::

            - AFG:RAMP:SYMmetry <NR3>
            - AFG:RAMP:SYMmetry?

        **Info:**
            - ``<NR3>`` is the floating point number that represents the AFG ramp symmetry, as a
              percentage.
        """
        return self._symmetry


class AfgPulseWidth(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:PULse:WIDth`` command.

    **Description:**
        - Sets (or queries) the AFG pulse width, in seconds.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:PULse:WIDth?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:PULse:WIDth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:PULse:WIDth value`` command.

    **SCPI Syntax:**

    ::

        - AFG:PULse:WIDth <NR3>
        - AFG:PULse:WIDth?

    **Info:**
        - ``<NR3>`` is the floating point number that represents the pulse width, in seconds.
    """


class AfgPulse(SCPICmdRead):
    """The ``AFG:PULse`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:PULse?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:PULse?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.width``: The ``AFG:PULse:WIDth`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._width = AfgPulseWidth(device, f"{self._cmd_syntax}:WIDth")

    @property
    def width(self) -> AfgPulseWidth:
        """Return the ``AFG:PULse:WIDth`` command.

        **Description:**
            - Sets (or queries) the AFG pulse width, in seconds.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:PULse:WIDth?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:PULse:WIDth?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:PULse:WIDth value`` command.

        **SCPI Syntax:**

        ::

            - AFG:PULse:WIDth <NR3>
            - AFG:PULse:WIDth?

        **Info:**
            - ``<NR3>`` is the floating point number that represents the pulse width, in seconds.
        """
        return self._width


class AfgPhase(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:PHASe`` command.

    **Description:**
        - Sets (or queries) the AFG phase. The AFG phase setting controls the phase difference
          between the trigger signal output and the AFG waveform output. Phase is expressed in
          degrees and ranges from -180.0 to 180.0 in increments of 0.1 degrees.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:PHASe?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:PHASe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:PHASe value`` command.

    **SCPI Syntax:**

    ::

        - AFG:PHASe <NR3>
        - AFG:PHASe?
    """


class AfgPeriod(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:PERIod`` command.

    **Description:**
        - Sets (or queries) the period of the AFG waveform, in seconds.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:PERIod?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:PERIod?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:PERIod value`` command.

    **SCPI Syntax:**

    ::

        - AFG:PERIod <NR3>
        - AFG:PERIod?

    **Info:**
        - ``NR3`` is the floating point number that represents the AFG period value, in seconds.
    """


class AfgOutputState(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:OUTPut:STATE`` command.

    **Description:**
        - Sets (or queries) the AFG output state.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:OUTPut:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:OUTPut:STATE value`` command.

    **SCPI Syntax:**

    ::

        - AFG:OUTPut:STATE {0|1|OFF|ON}
        - AFG:OUTPut:STATE?

    **Info:**
        - ``1`` or ON turns on the AFG output state.
        - ``0`` or OFF turns it off.
    """


class AfgOutputLoadImpedance(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:OUTPut:LOAd:IMPEDance`` command.

    **Description:**
        - Sets (or queries) the AFG output load impedance.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance value``
          command.

    **SCPI Syntax:**

    ::

        - AFG:OUTPut:LOAd:IMPEDance {FIFty|HIGHZ}
        - AFG:OUTPut:LOAd:IMPEDance?

    **Info:**
        - ``FIFty`` sets the output load impedance to 50 Ohms.
        - ``HIGHZ`` sets the output load impedance to the high-impedance state.
    """


class AfgOutputLoad(SCPICmdRead):
    """The ``AFG:OUTPut:LOAd`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.impedance``: The ``AFG:OUTPut:LOAd:IMPEDance`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._impedance = AfgOutputLoadImpedance(device, f"{self._cmd_syntax}:IMPEDance")

    @property
    def impedance(self) -> AfgOutputLoadImpedance:
        """Return the ``AFG:OUTPut:LOAd:IMPEDance`` command.

        **Description:**
            - Sets (or queries) the AFG output load impedance.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance value``
              command.

        **SCPI Syntax:**

        ::

            - AFG:OUTPut:LOAd:IMPEDance {FIFty|HIGHZ}
            - AFG:OUTPut:LOAd:IMPEDance?

        **Info:**
            - ``FIFty`` sets the output load impedance to 50 Ohms.
            - ``HIGHZ`` sets the output load impedance to the high-impedance state.
        """
        return self._impedance


class AfgOutput(SCPICmdRead):
    """The ``AFG:OUTPut`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:OUTPut?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.load``: The ``AFG:OUTPut:LOAd`` command tree.
        - ``.state``: The ``AFG:OUTPut:STATE`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._load = AfgOutputLoad(device, f"{self._cmd_syntax}:LOAd")
        self._state = AfgOutputState(device, f"{self._cmd_syntax}:STATE")

    @property
    def load(self) -> AfgOutputLoad:
        """Return the ``AFG:OUTPut:LOAd`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.impedance``: The ``AFG:OUTPut:LOAd:IMPEDance`` command.
        """
        return self._load

    @property
    def state(self) -> AfgOutputState:
        """Return the ``AFG:OUTPut:STATE`` command.

        **Description:**
            - Sets (or queries) the AFG output state.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:OUTPut:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:STATE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:OUTPut:STATE value`` command.

        **SCPI Syntax:**

        ::

            - AFG:OUTPut:STATE {0|1|OFF|ON}
            - AFG:OUTPut:STATE?

        **Info:**
            - ``1`` or ON turns on the AFG output state.
            - ``0`` or OFF turns it off.
        """
        return self._state


class AfgOffset(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:OFFSet`` command.

    **Description:**
        - Sets (or queries) the AFG offset value, in volts.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:OFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OFFSet?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:OFFSet value`` command.

    **SCPI Syntax:**

    ::

        - AFG:OFFSet <NR3>
        - AFG:OFFSet?

    **Info:**
        - ``<NR3>`` is a floating point number that represents the AFG offset, in volts.
    """


class AfgNoiseaddState(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:NOISEAdd:STATE`` command.

    **Description:**
        - Sets (or queries) the AFG additive noise state.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:NOISEAdd:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:STATE?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:STATE value`` command.

    **SCPI Syntax:**

    ::

        - AFG:NOISEAdd:STATE {0|1|OFF|ON}
        - AFG:NOISEAdd:STATE?

    **Info:**
        - ``1`` or ON turns on the AFG additive noise state.
        - ``0`` or OFF turns it off.
    """


class AfgNoiseaddPercent(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:NOISEAdd:PERCent`` command.

    **Description:**
        - Sets (or queries) the AFG additive noise level as a percentage. Minimum is 0.0%, maximum
          is 100.0% and increment is 1.0%.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:NOISEAdd:PERCent?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:PERCent?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:PERCent value`` command.

    **SCPI Syntax:**

    ::

        - AFG:NOISEAdd:PERCent <NR3>
        - AFG:NOISEAdd:PERCent?

    **Info:**
        - ``<NR3>`` is the floating point number that represents the AFG additive noise level, as a
          percentage.
    """


class AfgNoiseadd(SCPICmdRead):
    """The ``AFG:NOISEAdd`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:NOISEAdd?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.percent``: The ``AFG:NOISEAdd:PERCent`` command.
        - ``.state``: The ``AFG:NOISEAdd:STATE`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._percent = AfgNoiseaddPercent(device, f"{self._cmd_syntax}:PERCent")
        self._state = AfgNoiseaddState(device, f"{self._cmd_syntax}:STATE")

    @property
    def percent(self) -> AfgNoiseaddPercent:
        """Return the ``AFG:NOISEAdd:PERCent`` command.

        **Description:**
            - Sets (or queries) the AFG additive noise level as a percentage. Minimum is 0.0%,
              maximum is 100.0% and increment is 1.0%.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:NOISEAdd:PERCent?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:PERCent?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:PERCent value``
              command.

        **SCPI Syntax:**

        ::

            - AFG:NOISEAdd:PERCent <NR3>
            - AFG:NOISEAdd:PERCent?

        **Info:**
            - ``<NR3>`` is the floating point number that represents the AFG additive noise level,
              as a percentage.
        """
        return self._percent

    @property
    def state(self) -> AfgNoiseaddState:
        """Return the ``AFG:NOISEAdd:STATE`` command.

        **Description:**
            - Sets (or queries) the AFG additive noise state.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:NOISEAdd:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:STATE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:STATE value`` command.

        **SCPI Syntax:**

        ::

            - AFG:NOISEAdd:STATE {0|1|OFF|ON}
            - AFG:NOISEAdd:STATE?

        **Info:**
            - ``1`` or ON turns on the AFG additive noise state.
            - ``0`` or OFF turns it off.
        """
        return self._state


class AfgLowlevel(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:LOWLevel`` command.

    **Description:**
        - This command sets (or queries) the low level value of the output waveform, in volts, when
          using the arbitrary function generator feature.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:LOWLevel?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:LOWLevel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:LOWLevel value`` command.

    **SCPI Syntax:**

    ::

        - AFG:LOWLevel <NR3>
        - AFG:LOWLevel?

    **Info:**
        - ``<NR3>`` is the floating point number that represents the AFG low level value, in volts.
    """


class AfgLevelpreset(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:LEVELPreset`` command.

    **Description:**
        - Sets (or queries) the AFG preset levels to values that correspond to the logic standard
          specified by the argument. The presets set the following vertical controls: AMPLitude
          OFFSet HIGHLevel LOWLevel Note that once any of these vertical settings are changed from
          the preset values, or the output load impedance is changed, the query form returns USER.
          The ``AFG:LEVELPreset`` command sets the high level and low level values as follows:
          ``AFG:HIGHLevel`` and ``AFG:LOWLevel Setting`` Values Load Impedance FIFTY Ohm Load
          Impedance HIGHZ LEVEL Preset High Low High Low TTL N/A 5.0V ``CMOS_5_0V N``/A 5.0V
          ``CMOS_3_3V 2``.5V 0V 3.3V 0V ``CMOS_2_5V 2``.5V 0V 2.5V 0V ECL -0.85V -1.65V -0.9V -1.7V

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:LEVELPreset?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:LEVELPreset?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:LEVELPreset value`` command.

    **SCPI Syntax:**

    ::

        - AFG:LEVELPreset {CMOS_5_0V|CMOS_3_3V|CMOS_2_5V|ECL|TTL|USER}
        - AFG:LEVELPreset?

    **Info:**
        - ``CMOS_5_0V`` - standard 5-volt CMOS levels. Not available when the load impedance is 50
          Ohm.
        - ``CMOS_3_3V`` - standard 3.3-volt CMOS levels.
        - ``CMOS_2_5V`` - standard 2.5-volt CMOS levels.
        - ``USER`` - user-defined.
        - ``ECL`` - -1.7 to -0.9 volts (note the full range is not available in 50 Ohm - actual is
          -1.65 to -0.85. See table below.
        - ``TTL`` - 5.0 volts. Not available when the load impedance is 50 Ohm.
    """


class AfgHighlevel(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:HIGHLevel`` command.

    **Description:**
        - This command sets (or queries) the high level value of the output waveform, in volts, when
          using the arbitrary function generator feature.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:HIGHLevel?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:HIGHLevel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:HIGHLevel value`` command.

    **SCPI Syntax:**

    ::

        - AFG:HIGHLevel <NR3>
        - AFG:HIGHLevel?

    **Info:**
        - ``<NR3>`` is a floating point number that represents the AFG high level value, in volts.
    """


class AfgFunction(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:FUNCtion`` command.

    **Description:**
        - Sets (or queries) which AFG function to execute.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:FUNCtion?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:FUNCtion?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:FUNCtion value`` command.

    **SCPI Syntax:**

    ::

        - AFG:FUNCtion {SINE|SQUare|PULSe|RAMP|NOISe|DC|SINC|GAUSsian|LORENtz |ERISe|EDECAy|HAVERSINe|CARDIac|ARBitrary}
        - AFG:FUNCtion?

    **Info:**
        - ``SINE``
        - ``SQUare``
        - ``PULSe``
        - ``RAMP``
        - ``NOISe``
        - ``DC`` . The DC level is controlled by ``AFG:OFFSET``.
        - ``SINC`` (Sin(x)/x).
        - ``GAUSsian``
        - ``LORENtz``
        - ``ERISe``
        - ``EDECAy``
        - ``HAVERSINe``
        - ``CARDIac``
        - ``ARBitrary``
    """  # noqa: E501


class AfgFrequency(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:FREQuency`` command.

    **Description:**
        - Sets (or queries) the AFG frequency, in Hz.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:FREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:FREQuency?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:FREQuency value`` command.

    **SCPI Syntax:**

    ::

        - AFG:FREQuency <NR3>
        - AFG:FREQuency?

    **Info:**
        - ``<NR3>`` is the floating point number that represents the AFG frequency, in Hz.
    """


class AfgArbitraryEmemPointsEncdg(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:ARBitrary:EMEM:POINTS:ENCdg`` command.

    **Description:**
        - This command specifies the data encoding format for the ``AFG:ARBITRARY:EMEM:POINTS``
          query (either ASCII or binary). The default is ASCii. This setting is non-volatile and is
          reset by default setup or TekSecure. Refer to the ``AFG:ARBITRARY:EMEM:POINTS`` command
          description for more information.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:POINTS:ENCdg?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS:ENCdg?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS:ENCdg value``
          command.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:EMEM:POINTS:ENCdg {ASCii|BINary}
        - AFG:ARBitrary:EMEM:POINTS:ENCdg?

    **Info:**
        - ``ASCii`` - ASCII NR3 format.
        - ``BINary`` - IEEE488.2 binary block in 4-byte floating point format.
    """


class AfgArbitraryEmemPointsByteorder(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer`` command.

    **Description:**
        - ``:BYTEORDer This`` command specifies the byte order for the
          ``AFG:ARBITRARY:EMEM:POINTS?`` query when the ``AFG:ARBITRARY:EMEM:POINTS:ENCDG`` is set
          to BINary and when binary block data is sent for the  ``AFG:ARBITRARY:EMEM:POINTS``
          command. LSB - Least significant byte first (little endian) MSB - Most significant byte
          first (big endian) The default is LSB. This setting is non-volatile and is reset by
          default setup or TekSecure. Refer to the ``AFG:ARBITRARY:EMEM:POINTS`` command description
          for more information.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer?``
          query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer value`` command.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:EMEM:POINTS:BYTEORDer <LSB> |<MSB>
        - AFG:ARBitrary:EMEM:POINTS:BYTEORDer?

    **Info:**
        - ``LSB`` - Least significant byte first (little endian).
        - ``MSB`` - Most significant byte first (big endian).
    """


class AfgArbitraryEmemPoints(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:ARBitrary:EMEM:POINTS`` command.

    **Description:**
        - Specifies which points to load into the AFG arbitrary waveform edit memory. The point data
          to be loaded may be specified as an IEEE488.2 binary block with 4-byte floating point data
          values, or as a comma-separated list of NR2 or NR3 data values. The data values must be in
          the range of -1.0 to 1.0. The ``AFG:ARBITRARY:EMEM:POINTS:BYTEORDER`` command is used when
          the data is binary block format. The minimum number of points is 2 and maximum is 131072.
          Upon successful transfer of the data points, the ``AFG:ARBITRARY:EMEM:NUMPOINTS`` query
          will return the number of points loaded into arbitrary waveform edit memory and the
          ``AFG:ARBITRARY:EMEM:FUNCTION`` query will return USER. Note that the output, if turned
          on, will not change unless or until the AFG function is set to ARBitrary using the
          ``AFG:FUNCTION`` command. Point values may be coerced to the nearest valid step size. The
          query form returns the points stored in the AFG arbitrary waveform edit memory in the
          format specified by the ``AFG:ARBITRARY:EMEM:POINTS:ENCDG`` command and, for binary
          encoding, the byte order specified by the ``AFG:ARBITRARY:EMEM:POINTS:BYTEORDER`` command.
          Refer to the ``AFG:ARBITRARY:EMEM:POINTS:ENCDG`` command description for more information.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:POINTS?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS value``
          command.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:EMEM:POINTS <BlockWfmInDTO> |<NrfWfmInDTO>
        - AFG:ARBitrary:EMEM:POINTS?

    **Info:**
        - ``BlockWfmInDTO`` - an IEEE488.2 binary block with 4-byte floating point data values.
        - ``NrfWfmInDTO`` - a comma-separated list of NR2 or NR3 data values.

    Properties:
        - ``.byteorder``: The ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer`` command.
        - ``.encdg``: The ``AFG:ARBitrary:EMEM:POINTS:ENCdg`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._byteorder = AfgArbitraryEmemPointsByteorder(device, f"{self._cmd_syntax}:BYTEORDer")
        self._encdg = AfgArbitraryEmemPointsEncdg(device, f"{self._cmd_syntax}:ENCdg")

    @property
    def byteorder(self) -> AfgArbitraryEmemPointsByteorder:
        """Return the ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer`` command.

        **Description:**
            - ``:BYTEORDer This`` command specifies the byte order for the
              ``AFG:ARBITRARY:EMEM:POINTS?`` query when the ``AFG:ARBITRARY:EMEM:POINTS:ENCDG`` is
              set to BINary and when binary block data is sent for the
              ``AFG:ARBITRARY:EMEM:POINTS`` command. LSB - Least significant byte first (little
              endian) MSB - Most significant byte first (big endian) The default is LSB. This
              setting is non-volatile and is reset by default setup or TekSecure. Refer to the
              ``AFG:ARBITRARY:EMEM:POINTS`` command description for more information.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer value`` command.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:EMEM:POINTS:BYTEORDer <LSB> |<MSB>
            - AFG:ARBitrary:EMEM:POINTS:BYTEORDer?

        **Info:**
            - ``LSB`` - Least significant byte first (little endian).
            - ``MSB`` - Most significant byte first (big endian).
        """
        return self._byteorder

    @property
    def encdg(self) -> AfgArbitraryEmemPointsEncdg:
        """Return the ``AFG:ARBitrary:EMEM:POINTS:ENCdg`` command.

        **Description:**
            - This command specifies the data encoding format for the ``AFG:ARBITRARY:EMEM:POINTS``
              query (either ASCII or binary). The default is ASCii. This setting is non-volatile and
              is reset by default setup or TekSecure. Refer to the ``AFG:ARBITRARY:EMEM:POINTS``
              command description for more information.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:POINTS:ENCdg?``
              query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS:ENCdg?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AFG:ARBitrary:EMEM:POINTS:ENCdg value`` command.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:EMEM:POINTS:ENCdg {ASCii|BINary}
            - AFG:ARBitrary:EMEM:POINTS:ENCdg?

        **Info:**
            - ``ASCii`` - ASCII NR3 format.
            - ``BINary`` - IEEE488.2 binary block in 4-byte floating point format.
        """
        return self._encdg


class AfgArbitraryEmemNumpoints(SCPICmdRead):
    """The ``AFG:ARBitrary:EMEM:NUMPoints`` command.

    **Description:**
        - Returns the number of points in the AFB arbitrary waveform edit memory. This value will be
          used with the ``AFG:ARBITRARY:EMEM:GENERATE`` command in the event that the number of
          points is not specified. See the ``AFG:ARBITRARY:EMEM:GENERATE`` command description for
          more information.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:NUMPoints?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:NUMPoints?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:EMEM:NUMPoints?
    """


class AfgArbitraryEmemGenerate(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:ARBitrary:EMEM:GENerate`` command.

    **Description:**
        - This command generates the arbitrary waveform function specified by the enumeration
          argument. The NR1 argument is optional; it can be used to specify the number of points; if
          it is used, it also sets the value that will be used by ``AFG:ARBITRARY:EMEM:NUMPOINTS``
          until the instrument is reset via TEKSECURE. In the absence of the NR1 argument, the
          number of points used with the function is that number returned by the
          ``AFG:ARBITRARY:EMEM:NUMPOINTS`` query. To query the arbitrary waveform function set by
          this command, use ``AFG:ARBITRARY:EMEM:FUNCTION`` (This value is not reset by default
          setup or by power cycle).

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:GENerate?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:GENerate?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:EMEM:GENerate value``
          command.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:EMEM:GENerate {SINE|SQUare|PULSe|RAMP|NOISe[,NR1]}
        - AFG:ARBitrary:EMEM:GENerate?

    **Info:**
        - ``<NR1>`` (Optional) Specifies the number of points for the arbitrary waveform function.
          The number of points, if specified, must be >= 2 and <= 131072. The default number of
          points is 100.
        - ``SINE`` generates the Sine AFG function.
        - ``SQUare`` generates the Square AFG function.
        - ``PULSe`` generates the Pulse AFG function.
        - ``RAMP`` generates the Ramp AFG function.
        - ``NOISe`` generates the Noise AFG function.
    """


class AfgArbitraryEmemFunction(SCPICmdRead):
    """The ``AFG:ARBitrary:EMEM:FUNCtion`` command.

    **Description:**
        - Returns the currently selected arbitrary waveform pre-defined function. The pre-defined
          ARB function is selected using the command ``AFG:ARBITRARY:EMEM:GENERATE``. This query may
          also return USER, which indicates that the arbitrary waveform in edit memory has been
          altered from one of the predefined functions.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:FUNCtion?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:FUNCtion?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:EMEM:FUNCtion?
    """


class AfgArbitraryEmem(SCPICmdRead):
    """The ``AFG:ARBitrary:EMEM`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.function``: The ``AFG:ARBitrary:EMEM:FUNCtion`` command.
        - ``.generate``: The ``AFG:ARBitrary:EMEM:GENerate`` command.
        - ``.numpoints``: The ``AFG:ARBitrary:EMEM:NUMPoints`` command.
        - ``.points``: The ``AFG:ARBitrary:EMEM:POINTS`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._function = AfgArbitraryEmemFunction(device, f"{self._cmd_syntax}:FUNCtion")
        self._generate = AfgArbitraryEmemGenerate(device, f"{self._cmd_syntax}:GENerate")
        self._numpoints = AfgArbitraryEmemNumpoints(device, f"{self._cmd_syntax}:NUMPoints")
        self._points = AfgArbitraryEmemPoints(device, f"{self._cmd_syntax}:POINTS")

    @property
    def function(self) -> AfgArbitraryEmemFunction:
        """Return the ``AFG:ARBitrary:EMEM:FUNCtion`` command.

        **Description:**
            - Returns the currently selected arbitrary waveform pre-defined function. The
              pre-defined ARB function is selected using the command
              ``AFG:ARBITRARY:EMEM:GENERATE``. This query may also return USER, which indicates that
              the arbitrary waveform in edit memory has been altered from one of the predefined
              functions.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:FUNCtion?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:FUNCtion?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:EMEM:FUNCtion?
        """
        return self._function

    @property
    def generate(self) -> AfgArbitraryEmemGenerate:
        """Return the ``AFG:ARBitrary:EMEM:GENerate`` command.

        **Description:**
            - This command generates the arbitrary waveform function specified by the enumeration
              argument. The NR1 argument is optional; it can be used to specify the number of
              points; if it is used, it also sets the value that will be used by
              ``AFG:ARBITRARY:EMEM:NUMPOINTS`` until the instrument is reset via TEKSECURE. In the
              absence of the NR1 argument, the number of points used with the function is that
              number returned by the ``AFG:ARBITRARY:EMEM:NUMPOINTS`` query. To query the arbitrary
              waveform function set by this command, use ``AFG:ARBITRARY:EMEM:FUNCTION`` (This value
              is not reset by default setup or by power cycle).

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:GENerate?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:GENerate?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:EMEM:GENerate value``
              command.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:EMEM:GENerate {SINE|SQUare|PULSe|RAMP|NOISe[,NR1]}
            - AFG:ARBitrary:EMEM:GENerate?

        **Info:**
            - ``<NR1>`` (Optional) Specifies the number of points for the arbitrary waveform
              function. The number of points, if specified, must be >= 2 and <= 131072. The default
              number of points is 100.
            - ``SINE`` generates the Sine AFG function.
            - ``SQUare`` generates the Square AFG function.
            - ``PULSe`` generates the Pulse AFG function.
            - ``RAMP`` generates the Ramp AFG function.
            - ``NOISe`` generates the Noise AFG function.
        """
        return self._generate

    @property
    def numpoints(self) -> AfgArbitraryEmemNumpoints:
        """Return the ``AFG:ARBitrary:EMEM:NUMPoints`` command.

        **Description:**
            - Returns the number of points in the AFB arbitrary waveform edit memory. This value
              will be used with the ``AFG:ARBITRARY:EMEM:GENERATE`` command in the event that the
              number of points is not specified. See the ``AFG:ARBITRARY:EMEM:GENERATE`` command
              description for more information.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:NUMPoints?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:NUMPoints?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:EMEM:NUMPoints?
        """
        return self._numpoints

    @property
    def points(self) -> AfgArbitraryEmemPoints:
        """Return the ``AFG:ARBitrary:EMEM:POINTS`` command.

        **Description:**
            - Specifies which points to load into the AFG arbitrary waveform edit memory. The point
              data to be loaded may be specified as an IEEE488.2 binary block with 4-byte floating
              point data values, or as a comma-separated list of NR2 or NR3 data values. The data
              values must be in the range of -1.0 to 1.0. The
              ``AFG:ARBITRARY:EMEM:POINTS:BYTEORDER`` command is used when the data is binary block
              format. The minimum number of points is 2 and maximum is 131072. Upon successful
              transfer of the data points, the ``AFG:ARBITRARY:EMEM:NUMPOINTS`` query will return
              the number of points loaded into arbitrary waveform edit memory and the
              ``AFG:ARBITRARY:EMEM:FUNCTION`` query will return USER. Note that the output, if
              turned on, will not change unless or until the AFG function is set to ARBitrary using
              the ``AFG:FUNCTION`` command. Point values may be coerced to the nearest valid step
              size. The query form returns the points stored in the AFG arbitrary waveform edit
              memory in the format specified by the ``AFG:ARBITRARY:EMEM:POINTS:ENCDG`` command and,
              for binary encoding, the byte order specified by the
              ``AFG:ARBITRARY:EMEM:POINTS:BYTEORDER`` command. Refer to the
              ``AFG:ARBITRARY:EMEM:POINTS:ENCDG`` command description for more information.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM:POINTS?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:EMEM:POINTS value``
              command.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:EMEM:POINTS <BlockWfmInDTO> |<NrfWfmInDTO>
            - AFG:ARBitrary:EMEM:POINTS?

        **Info:**
            - ``BlockWfmInDTO`` - an IEEE488.2 binary block with 4-byte floating point data values.
            - ``NrfWfmInDTO`` - a comma-separated list of NR2 or NR3 data values.

        Sub-properties:
            - ``.byteorder``: The ``AFG:ARBitrary:EMEM:POINTS:BYTEORDer`` command.
            - ``.encdg``: The ``AFG:ARBitrary:EMEM:POINTS:ENCdg`` command.
        """
        return self._points


class AfgArbitraryArbItemTime(SCPICmdRead):
    """The ``AFG:ARBitrary:ARB<x>:TIMe`` command.

    **Description:**
        - Returns the time that the data in the specified arbitrary waveform slot was saved.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>:TIMe?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>:TIMe?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:ARB<x>:TIMe?
    """


class AfgArbitraryArbItemLabel(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:ARBitrary:ARB<x>:LABel`` command.

    **Description:**
        - Sets (or queries) the waveform label for arbitrary waveform slots 1- 4.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>:LABel?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:ARB<x>:LABel value``
          command.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:ARB<x>:LABel <QString>
        - AFG:ARBitrary:ARB<x>:LABel?
    """

    _WRAP_ARG_WITH_QUOTES = True


class AfgArbitraryArbItemDate(SCPICmdRead):
    """The ``AFG:ARBitrary:ARB<x>:DATE`` command.

    **Description:**
        - Returns the date that the data in the specified arbitrary waveform slot 1-4 was saved.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>:DATE?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>:DATE?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - AFG:ARBitrary:ARB<x>:DATE?
    """


class AfgArbitraryArbItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``AFG:ARBitrary:ARB<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.date``: The ``AFG:ARBitrary:ARB<x>:DATE`` command.
        - ``.label``: The ``AFG:ARBitrary:ARB<x>:LABel`` command.
        - ``.time``: The ``AFG:ARBitrary:ARB<x>:TIMe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._date = AfgArbitraryArbItemDate(device, f"{self._cmd_syntax}:DATE")
        self._label = AfgArbitraryArbItemLabel(device, f"{self._cmd_syntax}:LABel")
        self._time = AfgArbitraryArbItemTime(device, f"{self._cmd_syntax}:TIMe")

    @property
    def date(self) -> AfgArbitraryArbItemDate:
        """Return the ``AFG:ARBitrary:ARB<x>:DATE`` command.

        **Description:**
            - Returns the date that the data in the specified arbitrary waveform slot 1-4 was saved.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>:DATE?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>:DATE?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:ARB<x>:DATE?
        """
        return self._date

    @property
    def label(self) -> AfgArbitraryArbItemLabel:
        """Return the ``AFG:ARBitrary:ARB<x>:LABel`` command.

        **Description:**
            - Sets (or queries) the waveform label for arbitrary waveform slots 1- 4.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>:LABel?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:ARB<x>:LABel value``
              command.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:ARB<x>:LABel <QString>
            - AFG:ARBitrary:ARB<x>:LABel?
        """
        return self._label

    @property
    def time(self) -> AfgArbitraryArbItemTime:
        """Return the ``AFG:ARBitrary:ARB<x>:TIMe`` command.

        **Description:**
            - Returns the time that the data in the specified arbitrary waveform slot was saved.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>:TIMe?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>:TIMe?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - AFG:ARBitrary:ARB<x>:TIMe?
        """
        return self._time


class AfgArbitrary(SCPICmdRead):
    """The ``AFG:ARBitrary`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:ARBitrary?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.arb``: The ``AFG:ARBitrary:ARB<x>`` command tree.
        - ``.emem``: The ``AFG:ARBitrary:EMEM`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._arb: Dict[int, AfgArbitraryArbItem] = DefaultDictPassKeyToFactory(
            lambda x: AfgArbitraryArbItem(device, f"{self._cmd_syntax}:ARB{x}")
        )
        self._emem = AfgArbitraryEmem(device, f"{self._cmd_syntax}:EMEM")

    @property
    def arb(self) -> Dict[int, AfgArbitraryArbItem]:
        """Return the ``AFG:ARBitrary:ARB<x>`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:ARB<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:ARB<x>?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.date``: The ``AFG:ARBitrary:ARB<x>:DATE`` command.
            - ``.label``: The ``AFG:ARBitrary:ARB<x>:LABel`` command.
            - ``.time``: The ``AFG:ARBitrary:ARB<x>:TIMe`` command.
        """
        return self._arb

    @property
    def emem(self) -> AfgArbitraryEmem:
        """Return the ``AFG:ARBitrary:EMEM`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:EMEM?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:EMEM?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.function``: The ``AFG:ARBitrary:EMEM:FUNCtion`` command.
            - ``.generate``: The ``AFG:ARBitrary:EMEM:GENerate`` command.
            - ``.numpoints``: The ``AFG:ARBitrary:EMEM:NUMPoints`` command.
            - ``.points``: The ``AFG:ARBitrary:EMEM:POINTS`` command.
        """
        return self._emem


class AfgAmplitude(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:AMPLitude`` command.

    **Description:**
        - Sets (or queries) the AFG amplitude in volts, peak to peak.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG:AMPLitude?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:AMPLitude?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:AMPLitude value`` command.

    **SCPI Syntax:**

    ::

        - AFG:AMPLitude <NR3>
        - AFG:AMPLitude?

    **Info:**
        - ``<NR3>`` is a floating point number that represents the AFG amplitude, peak to peak, in
          volts.
    """


#  pylint: disable=too-many-instance-attributes
class Afg(SCPICmdRead):
    """The ``AFG`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AFG?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.amplitude``: The ``AFG:AMPLitude`` command.
        - ``.arbitrary``: The ``AFG:ARBitrary`` command tree.
        - ``.frequency``: The ``AFG:FREQuency`` command.
        - ``.function``: The ``AFG:FUNCtion`` command.
        - ``.highlevel``: The ``AFG:HIGHLevel`` command.
        - ``.levelpreset``: The ``AFG:LEVELPreset`` command.
        - ``.lowlevel``: The ``AFG:LOWLevel`` command.
        - ``.noiseadd``: The ``AFG:NOISEAdd`` command tree.
        - ``.offset``: The ``AFG:OFFSet`` command.
        - ``.output``: The ``AFG:OUTPut`` command tree.
        - ``.period``: The ``AFG:PERIod`` command.
        - ``.phase``: The ``AFG:PHASe`` command.
        - ``.pulse``: The ``AFG:PULse`` command tree.
        - ``.ramp``: The ``AFG:RAMP`` command tree.
        - ``.square``: The ``AFG:SQUare`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "AFG") -> None:
        super().__init__(device, cmd_syntax)
        self._amplitude = AfgAmplitude(device, f"{self._cmd_syntax}:AMPLitude")
        self._arbitrary = AfgArbitrary(device, f"{self._cmd_syntax}:ARBitrary")
        self._frequency = AfgFrequency(device, f"{self._cmd_syntax}:FREQuency")
        self._function = AfgFunction(device, f"{self._cmd_syntax}:FUNCtion")
        self._highlevel = AfgHighlevel(device, f"{self._cmd_syntax}:HIGHLevel")
        self._levelpreset = AfgLevelpreset(device, f"{self._cmd_syntax}:LEVELPreset")
        self._lowlevel = AfgLowlevel(device, f"{self._cmd_syntax}:LOWLevel")
        self._noiseadd = AfgNoiseadd(device, f"{self._cmd_syntax}:NOISEAdd")
        self._offset = AfgOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._output = AfgOutput(device, f"{self._cmd_syntax}:OUTPut")
        self._period = AfgPeriod(device, f"{self._cmd_syntax}:PERIod")
        self._phase = AfgPhase(device, f"{self._cmd_syntax}:PHASe")
        self._pulse = AfgPulse(device, f"{self._cmd_syntax}:PULse")
        self._ramp = AfgRamp(device, f"{self._cmd_syntax}:RAMP")
        self._square = AfgSquare(device, f"{self._cmd_syntax}:SQUare")

    @property
    def amplitude(self) -> AfgAmplitude:
        """Return the ``AFG:AMPLitude`` command.

        **Description:**
            - Sets (or queries) the AFG amplitude in volts, peak to peak.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:AMPLitude?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:AMPLitude?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:AMPLitude value`` command.

        **SCPI Syntax:**

        ::

            - AFG:AMPLitude <NR3>
            - AFG:AMPLitude?

        **Info:**
            - ``<NR3>`` is a floating point number that represents the AFG amplitude, peak to peak,
              in volts.
        """
        return self._amplitude

    @property
    def arbitrary(self) -> AfgArbitrary:
        """Return the ``AFG:ARBitrary`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:ARBitrary?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.arb``: The ``AFG:ARBitrary:ARB<x>`` command tree.
            - ``.emem``: The ``AFG:ARBitrary:EMEM`` command tree.
        """
        return self._arbitrary

    @property
    def frequency(self) -> AfgFrequency:
        """Return the ``AFG:FREQuency`` command.

        **Description:**
            - Sets (or queries) the AFG frequency, in Hz.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:FREQuency?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:FREQuency?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:FREQuency value`` command.

        **SCPI Syntax:**

        ::

            - AFG:FREQuency <NR3>
            - AFG:FREQuency?

        **Info:**
            - ``<NR3>`` is the floating point number that represents the AFG frequency, in Hz.
        """
        return self._frequency

    @property
    def function(self) -> AfgFunction:
        """Return the ``AFG:FUNCtion`` command.

        **Description:**
            - Sets (or queries) which AFG function to execute.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:FUNCtion?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:FUNCtion?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:FUNCtion value`` command.

        **SCPI Syntax:**

        ::

            - AFG:FUNCtion {SINE|SQUare|PULSe|RAMP|NOISe|DC|SINC|GAUSsian|LORENtz |ERISe|EDECAy|HAVERSINe|CARDIac|ARBitrary}
            - AFG:FUNCtion?

        **Info:**
            - ``SINE``
            - ``SQUare``
            - ``PULSe``
            - ``RAMP``
            - ``NOISe``
            - ``DC`` . The DC level is controlled by ``AFG:OFFSET``.
            - ``SINC`` (Sin(x)/x).
            - ``GAUSsian``
            - ``LORENtz``
            - ``ERISe``
            - ``EDECAy``
            - ``HAVERSINe``
            - ``CARDIac``
            - ``ARBitrary``
        """  # noqa: E501
        return self._function

    @property
    def highlevel(self) -> AfgHighlevel:
        """Return the ``AFG:HIGHLevel`` command.

        **Description:**
            - This command sets (or queries) the high level value of the output waveform, in volts,
              when using the arbitrary function generator feature.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:HIGHLevel?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:HIGHLevel?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:HIGHLevel value`` command.

        **SCPI Syntax:**

        ::

            - AFG:HIGHLevel <NR3>
            - AFG:HIGHLevel?

        **Info:**
            - ``<NR3>`` is a floating point number that represents the AFG high level value, in
              volts.
        """
        return self._highlevel

    @property
    def levelpreset(self) -> AfgLevelpreset:
        """Return the ``AFG:LEVELPreset`` command.

        **Description:**
            - Sets (or queries) the AFG preset levels to values that correspond to the logic
              standard specified by the argument. The presets set the following vertical controls:
              AMPLitude OFFSet HIGHLevel LOWLevel Note that once any of these vertical settings are
              changed from the preset values, or the output load impedance is changed, the query
              form returns USER. The ``AFG:LEVELPreset`` command sets the high level and low level
              values as follows: ``AFG:HIGHLevel`` and ``AFG:LOWLevel Setting`` Values Load
              Impedance FIFTY Ohm Load Impedance HIGHZ LEVEL Preset High Low High Low TTL N/A 5.0V
              ``CMOS_5_0V N``/A 5.0V ``CMOS_3_3V 2``.5V 0V 3.3V 0V ``CMOS_2_5V 2``.5V 0V 2.5V 0V ECL
              -0.85V -1.65V -0.9V -1.7V

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:LEVELPreset?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:LEVELPreset?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:LEVELPreset value`` command.

        **SCPI Syntax:**

        ::

            - AFG:LEVELPreset {CMOS_5_0V|CMOS_3_3V|CMOS_2_5V|ECL|TTL|USER}
            - AFG:LEVELPreset?

        **Info:**
            - ``CMOS_5_0V`` - standard 5-volt CMOS levels. Not available when the load impedance is
              50 Ohm.
            - ``CMOS_3_3V`` - standard 3.3-volt CMOS levels.
            - ``CMOS_2_5V`` - standard 2.5-volt CMOS levels.
            - ``USER`` - user-defined.
            - ``ECL`` - -1.7 to -0.9 volts (note the full range is not available in 50 Ohm - actual
              is -1.65 to -0.85. See table below.
            - ``TTL`` - 5.0 volts. Not available when the load impedance is 50 Ohm.
        """
        return self._levelpreset

    @property
    def lowlevel(self) -> AfgLowlevel:
        """Return the ``AFG:LOWLevel`` command.

        **Description:**
            - This command sets (or queries) the low level value of the output waveform, in volts,
              when using the arbitrary function generator feature.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:LOWLevel?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:LOWLevel?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:LOWLevel value`` command.

        **SCPI Syntax:**

        ::

            - AFG:LOWLevel <NR3>
            - AFG:LOWLevel?

        **Info:**
            - ``<NR3>`` is the floating point number that represents the AFG low level value, in
              volts.
        """
        return self._lowlevel

    @property
    def noiseadd(self) -> AfgNoiseadd:
        """Return the ``AFG:NOISEAdd`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:NOISEAdd?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.percent``: The ``AFG:NOISEAdd:PERCent`` command.
            - ``.state``: The ``AFG:NOISEAdd:STATE`` command.
        """
        return self._noiseadd

    @property
    def offset(self) -> AfgOffset:
        """Return the ``AFG:OFFSet`` command.

        **Description:**
            - Sets (or queries) the AFG offset value, in volts.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OFFSet?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:OFFSet value`` command.

        **SCPI Syntax:**

        ::

            - AFG:OFFSet <NR3>
            - AFG:OFFSet?

        **Info:**
            - ``<NR3>`` is a floating point number that represents the AFG offset, in volts.
        """
        return self._offset

    @property
    def output(self) -> AfgOutput:
        """Return the ``AFG:OUTPut`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:OUTPut?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.load``: The ``AFG:OUTPut:LOAd`` command tree.
            - ``.state``: The ``AFG:OUTPut:STATE`` command.
        """
        return self._output

    @property
    def period(self) -> AfgPeriod:
        """Return the ``AFG:PERIod`` command.

        **Description:**
            - Sets (or queries) the period of the AFG waveform, in seconds.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:PERIod?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:PERIod?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:PERIod value`` command.

        **SCPI Syntax:**

        ::

            - AFG:PERIod <NR3>
            - AFG:PERIod?

        **Info:**
            - ``NR3`` is the floating point number that represents the AFG period value, in seconds.
        """
        return self._period

    @property
    def phase(self) -> AfgPhase:
        """Return the ``AFG:PHASe`` command.

        **Description:**
            - Sets (or queries) the AFG phase. The AFG phase setting controls the phase difference
              between the trigger signal output and the AFG waveform output. Phase is expressed in
              degrees and ranges from -180.0 to 180.0 in increments of 0.1 degrees.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:PHASe?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:PHASe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:PHASe value`` command.

        **SCPI Syntax:**

        ::

            - AFG:PHASe <NR3>
            - AFG:PHASe?
        """
        return self._phase

    @property
    def pulse(self) -> AfgPulse:
        """Return the ``AFG:PULse`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:PULse?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:PULse?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.width``: The ``AFG:PULse:WIDth`` command.
        """
        return self._pulse

    @property
    def ramp(self) -> AfgRamp:
        """Return the ``AFG:RAMP`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:RAMP?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:RAMP?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.symmetry``: The ``AFG:RAMP:SYMmetry`` command.
        """
        return self._ramp

    @property
    def square(self) -> AfgSquare:
        """Return the ``AFG:SQUare`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AFG:SQUare?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:SQUare?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.duty``: The ``AFG:SQUare:DUty`` command.
        """
        return self._square



"""MSO6B device driver module."""
from tm_devices.commands import MSO6BMixin
from tm_devices.drivers.pi.scopes.tekscope.mso6 import MSO6


class MSO6B(MSO6BMixin, MSO6):
    """MSO6B device driver."""



"""The hardcopy commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - HARDCopy {STARt}
    - HARDCopy:ACTIVeprinter {<NR1>|<name>}
    - HARDCopy:ACTIVeprinter?
    - HARDCopy:INKSaver?
    - HARDCopy:LAYout {PORTRait|LANdscape}
    - HARDCopy:LAYout?
    - HARDCopy:PREVIEW {ON|OFF|<NR1>}
    - HARDCopy:PRINTer:ADD <name>,<server>,<address>|<emailaddress>
    - HARDCopy:PRINTer:DELete <name>
    - HARDCopy:PRINTer:LIST?
    - HARDCopy:PRINTer:REName <name>,<new_name>,<new_server>,<new_address>
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class HardcopyPrinterRename(SCPICmdWrite):
    """The ``HARDCopy:PRINTer:REName`` command.

    **Description:**
        - Renames a network or email printer on the list of available printers, replacing the
          currently stored settings with the settings specified in the command. The first argument
          can be either the printer name, or the index from querying ``HARDCOPY:PRINTER:LIST``

    **Usage:**
        - Using the ``.write(value)`` method will send the ``HARDCopy:PRINTer:REName value``
          command.

    **SCPI Syntax:**

    ::

        - HARDCopy:PRINTer:REName <name>,<new_name>,<new_server>,<new_address>

    **Info:**
        - ``Network printer:``
        - ``<index>`` is the name of the printer to be renamed (deleted).
        - ``<name>`` is the name of the printer to be renamed (deleted).
        - ``<new_printer_name>`` is the new name for this printer.
        - ``<new_server_name>`` is the new print server for this printer.
        - ``<new_server_address>`` is the new IP address for the server.
        - ``Email printer:``
        - ``<index>`` is the index or name of the printer to be renamed.
        - ``<name>`` is the index or name of the printer to be renamed.
        - ``<new printer email address`` > is the new email address for this printer.
    """


class HardcopyPrinterList(SCPICmdRead):
    """The ``HARDCopy:PRINTer:LIST`` command.

    **Description:**
        - Displays the list of currently defined printers. The fields for each entry represent the
          printer number, whether the printer is currently active (Y=active, N=inactive), the
          printer name, the printer type (USB, Net or Email), print server name or IP address.

    **Usage:**
        - Using the ``.query()`` method will send the ``HARDCopy:PRINTer:LIST?`` query.
        - Using the ``.verify(value)`` method will send the ``HARDCopy:PRINTer:LIST?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - HARDCopy:PRINTer:LIST?
    """


class HardcopyPrinterDelete(SCPICmdWrite):
    """The ``HARDCopy:PRINTer:DELete`` command.

    **Description:**
        - Removes a network printer from the list of available printers. The printer name is case
          sensitive.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``HARDCopy:PRINTer:DELete value``
          command.

    **SCPI Syntax:**

    ::

        - HARDCopy:PRINTer:DELete <name>

    **Info:**
        - ``<name>`` is the name of the printer to be deleted.
    """


class HardcopyPrinterAdd(SCPICmdWrite):
    """The ``HARDCopy:PRINTer:ADD`` command.

    **Description:**
        - This command is used to add a network or email printer to the list of available printers.
          Adding a network printer requires 3 arguments, but only one of server name or server IP
          address should be specified. An empty string can be used for blank arguments. Adding an
          email printer requires only one argument.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``HARDCopy:PRINTer:ADD value`` command.

    **SCPI Syntax:**

    ::

        - HARDCopy:PRINTer:ADD <name>,<server>,<address>|<emailaddress>

    **Info:**
        - ``Network printers:``
        - ``<name>`` is the name of the network printer queue.
        - ``<server>`` is the host name of the print (LPR) server.
        - ``<address>`` is the IP address of the print server.
        - ``Email printers:``
        - ``<emailaddress>`` is the internet email address of the printer. Must contain an @
          character.
    """


class HardcopyPrinter(SCPICmdRead):
    """The ``HARDCopy:PRINTer`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``HARDCopy:PRINTer?`` query.
        - Using the ``.verify(value)`` method will send the ``HARDCopy:PRINTer?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.add``: The ``HARDCopy:PRINTer:ADD`` command.
        - ``.delete``: The ``HARDCopy:PRINTer:DELete`` command.
        - ``.list``: The ``HARDCopy:PRINTer:LIST`` command.
        - ``.rename``: The ``HARDCopy:PRINTer:REName`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._add = HardcopyPrinterAdd(device, f"{self._cmd_syntax}:ADD")
        self._delete = HardcopyPrinterDelete(device, f"{self._cmd_syntax}:DELete")
        self._list = HardcopyPrinterList(device, f"{self._cmd_syntax}:LIST")
        self._rename = HardcopyPrinterRename(device, f"{self._cmd_syntax}:REName")

    @property
    def add(self) -> HardcopyPrinterAdd:
        """Return the ``HARDCopy:PRINTer:ADD`` command.

        **Description:**
            - This command is used to add a network or email printer to the list of available
              printers. Adding a network printer requires 3 arguments, but only one of server name
              or server IP address should be specified. An empty string can be used for blank
              arguments. Adding an email printer requires only one argument.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``HARDCopy:PRINTer:ADD value``
              command.

        **SCPI Syntax:**

        ::

            - HARDCopy:PRINTer:ADD <name>,<server>,<address>|<emailaddress>

        **Info:**
            - ``Network printers:``
            - ``<name>`` is the name of the network printer queue.
            - ``<server>`` is the host name of the print (LPR) server.
            - ``<address>`` is the IP address of the print server.
            - ``Email printers:``
            - ``<emailaddress>`` is the internet email address of the printer. Must contain an @
              character.
        """
        return self._add

    @property
    def delete(self) -> HardcopyPrinterDelete:
        """Return the ``HARDCopy:PRINTer:DELete`` command.

        **Description:**
            - Removes a network printer from the list of available printers. The printer name is
              case sensitive.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``HARDCopy:PRINTer:DELete value``
              command.

        **SCPI Syntax:**

        ::

            - HARDCopy:PRINTer:DELete <name>

        **Info:**
            - ``<name>`` is the name of the printer to be deleted.
        """
        return self._delete

    @property
    def list(self) -> HardcopyPrinterList:
        """Return the ``HARDCopy:PRINTer:LIST`` command.

        **Description:**
            - Displays the list of currently defined printers. The fields for each entry represent
              the printer number, whether the printer is currently active (Y=active, N=inactive),
              the printer name, the printer type (USB, Net or Email), print server name or IP
              address.

        **Usage:**
            - Using the ``.query()`` method will send the ``HARDCopy:PRINTer:LIST?`` query.
            - Using the ``.verify(value)`` method will send the ``HARDCopy:PRINTer:LIST?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - HARDCopy:PRINTer:LIST?
        """
        return self._list

    @property
    def rename(self) -> HardcopyPrinterRename:
        """Return the ``HARDCopy:PRINTer:REName`` command.

        **Description:**
            - Renames a network or email printer on the list of available printers, replacing the
              currently stored settings with the settings specified in the command. The first
              argument can be either the printer name, or the index from querying
              ``HARDCOPY:PRINTER:LIST``

        **Usage:**
            - Using the ``.write(value)`` method will send the ``HARDCopy:PRINTer:REName value``
              command.

        **SCPI Syntax:**

        ::

            - HARDCopy:PRINTer:REName <name>,<new_name>,<new_server>,<new_address>

        **Info:**
            - ``Network printer:``
            - ``<index>`` is the name of the printer to be renamed (deleted).
            - ``<name>`` is the name of the printer to be renamed (deleted).
            - ``<new_printer_name>`` is the new name for this printer.
            - ``<new_server_name>`` is the new print server for this printer.
            - ``<new_server_address>`` is the new IP address for the server.
            - ``Email printer:``
            - ``<index>`` is the index or name of the printer to be renamed.
            - ``<name>`` is the index or name of the printer to be renamed.
            - ``<new printer email address`` > is the new email address for this printer.
        """
        return self._rename


class HardcopyPreview(SCPICmdWrite):
    """The ``HARDCopy:PREVIEW`` command.

    **Description:**
        - Displays a preview of the current screen contents with the InkSaver Palette if
          ``HARDCopy:INKSAVER`` is 1 or the Normal Palette if ``HARDCopy:INKSAVER`` is 0.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``HARDCopy:PREVIEW value`` command.

    **SCPI Syntax:**

    ::

        - HARDCopy:PREVIEW {ON|OFF|<NR1>}

    **Info:**
        - ``ON`` or <NR1> ≠ 0 turns preview mode on.
        - ``OFF`` or <NR1> = 0 turns preview mode off.
    """


class HardcopyLayout(SCPICmdWrite, SCPICmdRead):
    """The ``HARDCopy:LAYout`` command.

    **Description:**
        - This command specifies the page orientation for hard copy. If you set the layout to
          LANdscape, the printer will print hard copies in landscape mode where the long edge of the
          screen will print to the long edge of the sheet of paper. If you set the layout to
          PORTRait, the printer will print hard copies in portrait mode. This command is not
          applicable for PictBridge hardcopies.

    **Usage:**
        - Using the ``.query()`` method will send the ``HARDCopy:LAYout?`` query.
        - Using the ``.verify(value)`` method will send the ``HARDCopy:LAYout?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HARDCopy:LAYout value`` command.

    **SCPI Syntax:**

    ::

        - HARDCopy:LAYout {PORTRait|LANdscape}
        - HARDCopy:LAYout?

    **Info:**
        - ``PORTRait`` orients the screen image vertically on the printed page.
        - ``LANdscape`` orients the screen image horizontally on the printed page.
    """


class HardcopyInksaver(SCPICmdRead):
    """The ``HARDCopy:INKSaver`` command.

    **Description:**
        - Changes hard copy output to print traces and graticule on a white background while
          retaining waveform color information (except for channel 1, which prints as dark blue
          because yellow does not show up well and is difficult to see on a white background). This
          option can significantly reduce print time and quantities of ink required compared with
          WYSIWYG dark background images.

    **Usage:**
        - Using the ``.query()`` method will send the ``HARDCopy:INKSaver?`` query.
        - Using the ``.verify(value)`` method will send the ``HARDCopy:INKSaver?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - HARDCopy:INKSaver?

    **Info:**
        - ``ON`` or <NR1> ≠ 0 sets the ink saver mode on.
        - ``OFF`` or <NR1> = 0 sets the ink saver mode off.
    """


class HardcopyActiveprinter(SCPICmdWrite, SCPICmdRead):
    """The ``HARDCopy:ACTIVeprinter`` command.

    **Description:**
        - This command specifies the currently active printer. When a hard copy operation is
          performed, the output will be sent to this printer. One of two methods of specifying the
          printer can be used: specifying an index value obtained from looking at the list of
          attached printers or by specifying the printer name.

    **Usage:**
        - Using the ``.query()`` method will send the ``HARDCopy:ACTIVeprinter?`` query.
        - Using the ``.verify(value)`` method will send the ``HARDCopy:ACTIVeprinter?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HARDCopy:ACTIVeprinter value`` command.

    **SCPI Syntax:**

    ::

        - HARDCopy:ACTIVeprinter {<NR1>|<name>}
        - HARDCopy:ACTIVeprinter?

    **Info:**
        - ``<NR1>`` is the index of the desired printer as returned from ``HARDCOPY:PRINTER:LIST``.
        - ``<name>`` is the name of the printer as specified in the printer list. This name is case
          sensitive and must be entered exactly as shown in the list.
    """


class Hardcopy(SCPICmdWrite, SCPICmdRead):
    """The ``HARDCopy`` command.

    **Description:**
        - Sends a hard copy of the screen display to the currently active printer using the current
          palette and layout settings.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``HARDCopy value`` command.

    **SCPI Syntax:**

    ::

        - HARDCopy {STARt}

    **Info:**
        - ``STARt`` sends a block of data representing the current screen image to the requested
          port. The data sent is in the image format specified by the ``SAVE:IMAGE:FILEFORMAT``
          command and the compression level is controlled by the selected format (BMP and TIFF are
          uncompressed whereas PNG is compressed). The ``HARDCOPY:INKSAVER`` determines whether the
          data sent is in InkSaver mode.

    Properties:
        - ``.activeprinter``: The ``HARDCopy:ACTIVeprinter`` command.
        - ``.inksaver``: The ``HARDCopy:INKSaver`` command.
        - ``.layout``: The ``HARDCopy:LAYout`` command.
        - ``.preview``: The ``HARDCopy:PREVIEW`` command.
        - ``.printer``: The ``HARDCopy:PRINTer`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "HARDCopy") -> None:
        super().__init__(device, cmd_syntax)
        self._activeprinter = HardcopyActiveprinter(device, f"{self._cmd_syntax}:ACTIVeprinter")
        self._inksaver = HardcopyInksaver(device, f"{self._cmd_syntax}:INKSaver")
        self._layout = HardcopyLayout(device, f"{self._cmd_syntax}:LAYout")
        self._preview = HardcopyPreview(device, f"{self._cmd_syntax}:PREVIEW")
        self._printer = HardcopyPrinter(device, f"{self._cmd_syntax}:PRINTer")

    @property
    def activeprinter(self) -> HardcopyActiveprinter:
        """Return the ``HARDCopy:ACTIVeprinter`` command.

        **Description:**
            - This command specifies the currently active printer. When a hard copy operation is
              performed, the output will be sent to this printer. One of two methods of specifying
              the printer can be used: specifying an index value obtained from looking at the list
              of attached printers or by specifying the printer name.

        **Usage:**
            - Using the ``.query()`` method will send the ``HARDCopy:ACTIVeprinter?`` query.
            - Using the ``.verify(value)`` method will send the ``HARDCopy:ACTIVeprinter?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HARDCopy:ACTIVeprinter value``
              command.

        **SCPI Syntax:**

        ::

            - HARDCopy:ACTIVeprinter {<NR1>|<name>}
            - HARDCopy:ACTIVeprinter?

        **Info:**
            - ``<NR1>`` is the index of the desired printer as returned from
              ``HARDCOPY:PRINTER:LIST``.
            - ``<name>`` is the name of the printer as specified in the printer list. This name is
              case sensitive and must be entered exactly as shown in the list.
        """
        return self._activeprinter

    @property
    def inksaver(self) -> HardcopyInksaver:
        """Return the ``HARDCopy:INKSaver`` command.

        **Description:**
            - Changes hard copy output to print traces and graticule on a white background while
              retaining waveform color information (except for channel 1, which prints as dark blue
              because yellow does not show up well and is difficult to see on a white background).
              This option can significantly reduce print time and quantities of ink required
              compared with WYSIWYG dark background images.

        **Usage:**
            - Using the ``.query()`` method will send the ``HARDCopy:INKSaver?`` query.
            - Using the ``.verify(value)`` method will send the ``HARDCopy:INKSaver?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - HARDCopy:INKSaver?

        **Info:**
            - ``ON`` or <NR1> ≠ 0 sets the ink saver mode on.
            - ``OFF`` or <NR1> = 0 sets the ink saver mode off.
        """
        return self._inksaver

    @property
    def layout(self) -> HardcopyLayout:
        """Return the ``HARDCopy:LAYout`` command.

        **Description:**
            - This command specifies the page orientation for hard copy. If you set the layout to
              LANdscape, the printer will print hard copies in landscape mode where the long edge of
              the screen will print to the long edge of the sheet of paper. If you set the layout to
              PORTRait, the printer will print hard copies in portrait mode. This command is not
              applicable for PictBridge hardcopies.

        **Usage:**
            - Using the ``.query()`` method will send the ``HARDCopy:LAYout?`` query.
            - Using the ``.verify(value)`` method will send the ``HARDCopy:LAYout?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HARDCopy:LAYout value`` command.

        **SCPI Syntax:**

        ::

            - HARDCopy:LAYout {PORTRait|LANdscape}
            - HARDCopy:LAYout?

        **Info:**
            - ``PORTRait`` orients the screen image vertically on the printed page.
            - ``LANdscape`` orients the screen image horizontally on the printed page.
        """
        return self._layout

    @property
    def preview(self) -> HardcopyPreview:
        """Return the ``HARDCopy:PREVIEW`` command.

        **Description:**
            - Displays a preview of the current screen contents with the InkSaver Palette if
              ``HARDCopy:INKSAVER`` is 1 or the Normal Palette if ``HARDCopy:INKSAVER`` is 0.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``HARDCopy:PREVIEW value`` command.

        **SCPI Syntax:**

        ::

            - HARDCopy:PREVIEW {ON|OFF|<NR1>}

        **Info:**
            - ``ON`` or <NR1> ≠ 0 turns preview mode on.
            - ``OFF`` or <NR1> = 0 turns preview mode off.
        """
        return self._preview

    @property
    def printer(self) -> HardcopyPrinter:
        """Return the ``HARDCopy:PRINTer`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``HARDCopy:PRINTer?`` query.
            - Using the ``.verify(value)`` method will send the ``HARDCopy:PRINTer?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.add``: The ``HARDCopy:PRINTer:ADD`` command.
            - ``.delete``: The ``HARDCopy:PRINTer:DELete`` command.
            - ``.list``: The ``HARDCopy:PRINTer:LIST`` command.
            - ``.rename``: The ``HARDCopy:PRINTer:REName`` command.
        """
        return self._printer
